@@ -1,7 +1,4 @@
-"""
-
-A SubFS represents a directory in a 'parent' filesystem.
-
+"""Manage a directory in a *parent* filesystem.
 """
 
 from __future__ import print_function
@@ -15,12 +12,11 @@ from .path import abspath, join, normpath, relpath
 
 @six.python_2_unicode_compatible
 class SubFS(WrapFS):
-    """
-    A sub-directory on another filesystem.
+    """A sub-directory on another filesystem.
 
     A SubFS is a filesystem object that maps to a sub-directory of
     another filesystem. This is the object that is returned by
-    :meth:`~fs.base.FS.opendir`.
+    `~fs.base.FS.opendir`.
 
     """
 
@@ -29,7 +25,8 @@ class SubFS(WrapFS):
         self._sub_dir = abspath(normpath(path))
 
     def __repr__(self):
-        return "SubFS({!r}, {!r})".format(
+        return "{}({!r}, {!r})".format(
+            self.__class__.__name__,
             self._wrap_fs,
             self._sub_dir
         )
@@ -46,3 +43,12 @@ class SubFS(WrapFS):
     def delegate_path(self, path):
         _path = join(self._sub_dir, relpath(normpath(path)))
         return self._wrap_fs, _path
+
+
+class ClosingSubFS(SubFS):
+    """A version of `SubFS` which closes its parent when closed.
+    """
+
+    def close(self):
+        self.delegate_fs().close()
+        super(ClosingSubFS, self).close()
